@@ -17,12 +17,14 @@ This currently supports:
  * Desktop files (*.desktop.in) [into prefix/share/applications, or
    prefix/share/autostart if they have "autostart" anywhere in the path]
  * KDE4 notifications (*.notifyrc.in)
+ * Apport hooks (apport/*) [installed into /usr/share/apport/package-hooks]
  * scripts (all in bin/, and ./<projectname>
  * Auxiliary data files (in data/*) [into prefix/share/<projectname>/]
  * automatic po/POTFILES.in (with all source files which contain _())
  * automatic MANIFEST (everything except swap and backup files, *.pyc, and
    revision control)
  * manpages (*.[0-9])
+ * icons (data/icons/<size>/<category>/*.{svg,png})
  * files which should go into /etc (./etc/*, copied verbatim)
  * determining "requires" from import statements in source code
  * determining "provides" from shipped packages and modules
@@ -32,15 +34,14 @@ If you follow above conventions, then you don't need any po/POTFILES.in,
 author, license, etc.) in ./setup.py.
 '''
 
-__version__ = '2.12'
-
 # (c) 2009 Canonical Ltd.
 # Author: Martin Pitt <martin.pitt@ubuntu.com>
 
 import os, os.path, fnmatch, stat, sys, subprocess
-import compiler # TODO: deprecated
+import ast
 import distutils.core
 
+from DistUtilsExtra import __version__ as __pkgversion
 from DistUtilsExtra.command import *
 import distutils.dir_util
 import distutils.command.clean
@@ -48,6 +49,8 @@ import distutils.command.sdist
 import distutils.command.install
 import distutils.filelist
 
+__version__ = __pkgversion
+
 # FIXME: global variable, to share with build_i18n_auto
 src = {}
 src_all = {}
@@ -68,7 +71,8 @@ def setup(**attrs):
 
     # mark files in etc/*, handled by install_auto
     # don't install DistUtilsExtra if bundled with a source tarball
-    ignore_dirs = ['etc', 'DistUtilsExtra']
+    # ignore packaging
+    ignore_dirs = ['etc', 'DistUtilsExtra', 'debian']
     
     for f in src.copy():
         for d in ignore_dirs:
@@ -80,6 +84,7 @@ def setup(**attrs):
     __packages(attrs, src)
     __provides(attrs, src)
     __dbus(attrs, src)
+    __apport_hooks(attrs, src)
     __data(attrs, src)
     __scripts(attrs, src)
     __stdfiles(attrs, src)
@@ -189,6 +194,20 @@ def __dbus(attrs, src):
     if session_service:
         v.append(('share/dbus-1/services', session_service))
 
+def __apport_hooks(attrs, src):      
+    '''Apport hooks'''  
+    v = attrs.setdefault('data_files', [])
+
+    # files will be copied to /usr/share/apport/package-hooks/
+    hooks = []
+    assert 'name' in attrs, 'You need to set the "name" property in setup.py'
+    for f in src_fileglob(src, '*.py'):
+        if f.startswith('apport/'):
+            hooks.append(f)
+            src_mark(src, f)
+    if hooks:
+        v.append(('share/apport/package-hooks/', hooks))
+
 def __data(attrs, src):
     '''Install auxiliary data files.
 
@@ -203,7 +222,9 @@ def __data(attrs, src):
     for f in src.copy():
         if f.startswith('data/') and not f.startswith('data/icons/') and \
                 not f.endswith('.desktop.in') and not f.endswith('*.notifyrc.in'):
-            v.append((os.path.join('share', attrs['name'], os.path.dirname(f[5:])), [f]))
+            if not os.path.islink(f):
+                # symlinks are handled in install_auto
+                v.append((os.path.join('share', attrs['name'], os.path.dirname(f[5:])), [f]))
             src_mark(src, f)
 
 def __scripts(attrs, src):
@@ -218,10 +239,13 @@ def __scripts(attrs, src):
     scripts = []
     for f in src.copy():
         if f.startswith('bin/') or f == attrs['name']:
-            st = os.stat(f)
+            st = os.lstat(f)
             if stat.S_ISREG(st.st_mode) and st.st_mode & stat.S_IEXEC:
                 scripts.append(f)
                 src_mark(src, f)
+            elif stat.S_ISLNK(st.st_mode):
+                # symlinks are handled in install_auto
+                src_mark(src, f)
 
     if scripts:
         v = attrs.setdefault('scripts', [])
@@ -306,16 +330,16 @@ def __add_imports(imports, file, attrs):
     This filters out modules which are shipped with Python itself.
     '''
     try:
-        ast = compiler.parseFile(file)
+        tree = ast.parse(open(file).read(), file)
 
-        for node in ast.node.nodes:
-            if isinstance(node, compiler.ast.Import):
-                for name, _ in node.names:
-                    if __external_mod(name, attrs):
-                        imports.add(name)
-            if isinstance(node, compiler.ast.From):
-                if __external_mod(node.modname, attrs):
-                    imports.add(node.modname)
+        for node in ast.walk(tree):
+            if isinstance(node, ast.Import):
+                for alias in node.names:
+                    if __external_mod(alias.name, attrs):
+                        imports.add(alias.name)
+            if isinstance(node, ast.ImportFrom):
+                if __external_mod(node.module, attrs):
+                    imports.add(node.module)
     except SyntaxError, e:
         print >> sys.stderr, 'WARNING: syntax errors in', file, ':', e
 
@@ -519,45 +543,46 @@ class build_i18n_auto(build_i18n.build_i18n):
         auto_potfiles_in = False
         exe_symlinks = []
         global src_all
-        if not os.path.exists(os.path.join('po', 'POTFILES.in')):
-            files = src_fileglob(src_all, '*.py')
-            files.update(src_fileglob(src_all, '*.desktop.in'))
-            files.update(src_fileglob(src_all, '*.notifyrc.in'))
-            files.update(src_fileglob(src_all, '*.policy.in'))
+        try:
+            if not os.path.exists(os.path.join('po', 'POTFILES.in')):
+                files = src_fileglob(src_all, '*.py')
+                files.update(src_fileglob(src_all, '*.desktop.in'))
+                files.update(src_fileglob(src_all, '*.notifyrc.in'))
+                files.update(src_fileglob(src_all, '*.policy.in'))
 
-            for f in src_fileglob(src_all, '*.ui'):
-                contents = open(f).read()
-                if ('<interface>\n' in contents or '<interface ' in contents) and 'class="Gtk' in contents:
-                    files.add('[type: gettext/glade]' + f)
+                for f in src_fileglob(src_all, '*.ui'):
+                    contents = open(f).read()
+                    if ('<interface>\n' in contents or '<interface ' in contents) and 'class="Gtk' in contents:
+                        files.add('[type: gettext/glade]' + f)
 
-            # find extensionless executable scripts which are Python files, and
-            # generate a temporary *.py alias, so that they get caught by
-            # intltool
-            for f in src_all:
-                f_py = f + '.py'
-                if os.access(f, os.X_OK) and os.path.splitext(f)[1] == '' and \
-                        not os.path.exists(f_py):
-                    line = open(f).readline()
-                    if line.startswith('#!') and 'python' in line:
-                        os.symlink(os.path.basename(f), f_py)
-                        files.add(f_py)
-                        exe_symlinks.append(f_py)
+                # find extensionless executable scripts which are Python files, and
+                # generate a temporary *.py alias, so that they get caught by
+                # intltool
+                for f in reduce(lambda x, y: x.union(y[1]), self.distribution.data_files, src_all):
+                    f_py = f + '.py'
+                    if os.access(f, os.X_OK) and os.path.splitext(f)[1] == '' and \
+                            not os.path.exists(f_py):
+                        line = open(f).readline()
+                        if line.startswith('#!') and 'python' in line:
+                            os.symlink(os.path.basename(f), f_py)
+                            files.add(f_py)
+                            exe_symlinks.append(f_py)
 
-            if files:
-                if not os.path.isdir('po'):
-                    os.mkdir('po')
-                potfiles_in = open('po/POTFILES.in', 'w')
-                print >> potfiles_in, '[encoding: UTF-8]'
-                for f in files:
-                    print >> potfiles_in, f
-                potfiles_in.close()
+                if files:
+                    if not os.path.isdir('po'):
+                        os.mkdir('po')
+                    potfiles_in = open('po/POTFILES.in', 'w')
+                    print >> potfiles_in, '[encoding: UTF-8]'
+                    for f in files:
+                        print >> potfiles_in, f
+                    potfiles_in.close()
 
-                auto_potfiles_in = True
+                    auto_potfiles_in = True
 
-        build_i18n.build_i18n.run(self)
-
-        for f in exe_symlinks:
-            os.unlink(f)
+            build_i18n.build_i18n.run(self)
+        finally:
+            for f in exe_symlinks:
+                os.unlink(f)
 
         if auto_potfiles_in:
             os.unlink('po/POTFILES.in')
@@ -573,9 +598,13 @@ class build_kdeui_auto(build_kdeui.build_kdeui):
         # add *.ui files which belong to KDE4
         kdeui_files = []
         for f in src_fileglob(src, '*.ui'):
-            if open(f).readline().startswith('<ui version="'):
+            fd = open(f)
+            # might be on the first or second line
+            if fd.readline().startswith('<ui version="') or \
+               fd.readline().startswith('<ui version="'):
                 src_mark(src, f)
                 kdeui_files.append(f)
+            fd.close()
         if kdeui_files:
             try:
                 uf = eval(self.ui_files)
@@ -625,7 +654,7 @@ class sdist_auto(distutils.command.sdist.sdist):
             self.filelist.append(f)
 
 #
-# Automatic installation of ./etc/
+# Automatic installation of ./etc/ and symlinks
 #
 
 class install_auto(distutils.command.install.install):
@@ -634,16 +663,39 @@ class install_auto(distutils.command.install.install):
         if os.path.isdir('etc'):
             # work around a bug in copy_tree() which fails with "File exists" on
             # previously existing symlinks
-            for f in distutils.filelist.findall():
+            for f in distutils.filelist.findall('etc'):
                 if not f.startswith('etc' + os.path.sep) or not os.path.islink(f):
                     continue
                 try:
                     os.unlink(os.path.join(self.root, f))
                 except OSError:
                     pass
-
+            if not self.root:
+                self.root = ''
             distutils.dir_util.copy_tree('etc', os.path.join(self.root, 'etc'),
                     preserve_times=0, preserve_symlinks=1, verbose=1)
 
+        # install data/scripts symlinks
+        for f in distutils.filelist.findall():
+            if not os.path.islink(f):
+                continue
+            if f.startswith('bin/') or f.startswith('data/'):
+                if f.startswith('bin'):
+                    dir = self.install_scripts
+                    dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[1:]))
+                elif f.startswith('data/icons'):
+                    dir = os.path.join(self.install_data, 'share', 'icons', 'hicolor')
+                    dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[2:]))
+                else:
+                    dir = os.path.join(self.install_data, 'share', self.distribution.get_name())
+                    dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[1:]))
+
+                d = os.path.dirname(dest)
+                if not os.path.isdir(d):
+                    os.makedirs(d)
+                if os.path.exists(dest):
+                    os.unlink(dest)
+                os.symlink(os.readlink(f), dest)
+
         distutils.command.install.install.run(self)
 
