@@ -2,7 +2,7 @@
 
 # test DistUtilsExtra.auto
 
-import sys, unittest, shutil, tempfile, os, os.path, subprocess
+import sys, unittest, shutil, tempfile, os, os.path, subprocess, re
 
 class T(unittest.TestCase):
     def setUp(self):
@@ -28,7 +28,6 @@ setup(
         try:
             # check that setup.py clean removes everything
             (o, e, s) = self.setup_py(['clean', '-a'])
-            self.assertEqual(e, '')
             self.assertEqual(s, 0)
             cruft = self.diff_snapshot()
             self.assertEqual(cruft, '', 'no cruft after cleaning:\n' + cruft)
@@ -192,7 +191,7 @@ def add_info(report):
             os.path.join(self.install_tree,
             'usr/share/locale/de/LC_MESSAGES/foo.mo')],
             stdout=subprocess.PIPE)
-        out = msgunfmt.communicate()[0]
+        out = msgunfmt.communicate()[0].decode()
         self.assertEqual(out, open(os.path.join(self.src, 'po/de.po')).read())
 
     def test_policykit(self):
@@ -330,8 +329,8 @@ setup(
 ''')
 
         self._mksrc('data/stuff')
-        self._mksrc('data/handlers/red.py', 'import sys\nprint "RED"')
-        self._mksrc('data/handlers/blue.py', 'import sys\nprint "BLUE"')
+        self._mksrc('data/handlers/red.py', 'import sys\nprint ("RED")')
+        self._mksrc('data/handlers/blue.py', 'import sys\nprint ("BLUE")')
         self._mksrc('data/40-foo.rules')
         self._mksrc('data/blob1.conf')
         self._mksrc('data/blob2.conf')
@@ -454,11 +453,11 @@ gui/foo.desktop.in
         self._mk_i18n_source()
 
         # add some additional binaries here which aren't caught by default
-        self._mksrc('cli/client-cli', "#!/usr/bin/python\nprint _('yes15')", True)
-        self._mksrc('gtk/client-gtk', '#!/usr/bin/python\nprint _("yes16")', True)
+        self._mksrc('cli/client-cli', "#!/usr/bin/python\nprint (_('yes15'))", True)
+        self._mksrc('gtk/client-gtk', '#!/usr/bin/python\nprint (_("yes16"))', True)
         # this is the most tricky case: intltool doesn't consider them Python
         # files by default and thus just looks for _(""):
-        self._mksrc('kde/client-kde', "#!/usr/bin/python\nprint _('yes17')", True)
+        self._mksrc('kde/client-kde', "#!/usr/bin/python\nprint (_('yes17'))", True)
         self._mksrc('po/POTFILES.in.in', 'gtk/client-gtk\nkde/client-kde')
         self._mksrc('setup.py', '''
 from DistUtilsExtra.auto import setup
@@ -652,16 +651,16 @@ setup(
         '''automatic requires/provides'''
 
         try:
-            __import__('xdg')
-            __import__('dateutil')
+            __import__('pkg_resources')
+            __import__('httplib2')
         except ImportError:
-            self.fail('You need to have xdg and dateutil installed for this test suite to work')
+            self.fail('You need to have pkg_resources and httplib2 installed for this test suite to work')
 
         self._mksrc('foo/__init__.py', '')
         self._mksrc('foo/stuff.py', '''import xml.parsers.expat
 import os, os.path, email.mime, distutils.command.register
 from email import header as h
-import xdg.Mime, unknown
+import httplib2.iri2uri, unknown
 ''')
 
         self._mksrc('foo/bar/__init__.py', '')
@@ -671,11 +670,11 @@ import xdg.Mime, unknown
 
         self._mksrc('bin/foo-cli', '''#!/usr/bin/python
 import sys
-from dateutil import tz
+import pkg_resources
 import foo.bar
-from xdg import Menu
+from httplib2 import iri2uri
 
-print 'import iamnota.module'
+print ('import iamnota.module')
 ''', executable=True)
 
         # this shouldn't be treated specially
@@ -708,7 +707,7 @@ print 'import iamnota.module'
 
         # check requires
         req = [prop.split(' ', 1)[1] for prop in egg if prop.startswith('Requires: ')]
-        self.assertEqual(set(req), set(['DistUtilsExtra.auto', 'xdg', 'dateutil']))
+        self.assertEqual(set(req), set(['DistUtilsExtra.auto', 'httplib2', 'pkg_resources']))
 
     def test_help(self):
         '''Docbook XML help'''
@@ -758,12 +757,22 @@ print 'import iamnota.module'
 
         env = os.environ
         oldcwd = os.getcwd()
-        env['PYTHONPATH'] = oldcwd
+        if 'PYTHONPATH' not in env:
+            env['PYTHONPATH'] = oldcwd
         os.chdir(self.src)
-        s = subprocess.Popen(['python', 'setup.py'] + args, env=env,
+        s = subprocess.Popen(['/proc/self/exe', 'setup.py'] + args, env=env,
             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         (out, err) = s.communicate()
+        out = out.decode()
+        err = err.decode()
         os.chdir(oldcwd)
+
+        # python3 distutils bug workaround, filter out bogus errors
+        bogus_re = re.compile('^file .* \(for module .*\) not found$')
+        err = '\n'.join([l for l in err.splitlines() if not bogus_re.match(l)])
+        if err:
+            err += '\n'
+
         return (out, err, s.returncode)
 
     def do_install(self):
@@ -797,13 +806,13 @@ print 'import iamnota.module'
         f = open(path, 'w')
         if content is None:
             # default content, to spot with diff
-            print >> f, 'dummy'
+            f.write('dummy')
         else:
-            print >> f, content
+            f.write(content + '\n')
         f.close()
 
         if executable:
-            os.chmod(path, 0755)
+            os.chmod(path, 0o755)
 
     def do_snapshot(self):
         '''Snapshot source tree.
@@ -824,7 +833,8 @@ print 'import iamnota.module'
         diff = subprocess.Popen(['diff', '-x', 'foo.pot', '-Nur', os.path.join(self.snapshot, 's'), 
             self.src], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         (out, err) = diff.communicate()
-        self.assertEqual(err, '', 'diff error messages')
+        out = out.decode()
+        self.assertEqual(err, b'', 'diff error messages')
         return out
 
     def _mkpo(self):
@@ -848,26 +858,26 @@ msgstr "Bonjour"''')
     def _mk_i18n_source(self):
         '''Create some example source files with gettext calls'''
 
-        self._mksrc('gtk/main.py', '''print _("yes1")
-print "no1"
-print __("no2")
+        self._mksrc('gtk/main.py', '''print (_("yes1"))
+print ("no1")
+print (__("no2"))
 x = _('yes2 %s') % y
 
 def f():
-    print _(u"yes3")
-    return _(u'yes6')''')
+    print (_("yes3"))
+    return _('yes6')''')
 
         self._mksrc('helpers.py', '''
-print f(_(u"yes4"))
-print _(\'\'\'yes5
+print (f(_("yes4")))
+print (_(\'\'\'yes5
 even more
-lines\'\'\')
-print _("""yes6
-more lines""")
-print \'\'\'no3
-boo\'\'\'
-print """no4
-more"""''')
+lines\'\'\'))
+print (_("""yes6
+more lines"""))
+print (\'\'\'no3
+boo\'\'\')
+print ("""no4
+more""")''')
 
         self._mksrc('gui/foo.desktop.in', '''[Desktop Entry]
 _Name=yes7
@@ -910,9 +920,9 @@ Exec=/usr/bin/foo''')
         self._mksrc('Makefile', 'echo _("no7")')
 
         # Executables without *.py extension
-        self._mksrc('gtk/foo-gtk', '#!/usr/bin/python\nprint _("yes13")',
+        self._mksrc('gtk/foo-gtk', '#!/usr/bin/python\nprint (_("yes13"))',
                 executable=True)
-        self._mksrc('cli/foo-cli', '#!/usr/bin/env python\nprint _(\'yes14\')',
+        self._mksrc('cli/foo-cli', '#!/usr/bin/env python\nprint (_(\'yes14\'))',
                 executable=True)
         self._mksrc('daemon/foobarize', '#!/usr/bin/flex\np _("no8")',
                 executable=True)
